(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('@angular/core'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('cobaui', ['exports', 'rxjs', '@angular/core', 'rxjs/operators'], factory) :
    (factory((global.cobaui = {}),global.rxjs,global.ng.core,global.rxjs.operators));
}(this, (function (exports,rxjs,core,operators) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * The Adaptation Controller is the core component of an adaptive UI.
     * It receives context information from Context Providers and evaluates Adaptation Rules via a Rule Engine session
     * to trigger adaptations on Adaptive Widgets in the UI.
     */
    var AdaptationController = /** @class */ (function () {
        /**
         * Constructor: Inject Angulars Dependency Injector
         */
        function AdaptationController(injector) {
            /**
             * Keeps track of registered context provider components
             */
            this.contextProviders = [];
            /**
             * Keeps track of registered rule provider components
             */
            this.ruleProviders = [];
            /**
             * Observable stream of the context of use
             */
            this.context$ = new rxjs.BehaviorSubject([]);
            /**
             * Observable stream of adaptation rules
             */
            this.rules$ = new rxjs.BehaviorSubject([]);
            /**
             * Observable stream of Adaptation Actions
             */
            this.adaptationActions$ = new rxjs.Subject();
            /**
             * Keeps track of subscriptions to observables
             */
            this.cpSubscription = new rxjs.Subscription();
            this.rpSubscription = new rxjs.Subscription();
            this.injector = injector;
        }
        /**
         * Register a Rule Evaluator component
         * @param ruleEvaluatorToken
         */
        /**
         * Register a Rule Evaluator component
         * @param {?} ruleEvaluatorToken
         * @return {?}
         */
        AdaptationController.prototype.registerRuleEvaluator = /**
         * Register a Rule Evaluator component
         * @param {?} ruleEvaluatorToken
         * @return {?}
         */
            function (ruleEvaluatorToken) {
                this.ruleEvaluator = this.injector.get(ruleEvaluatorToken);
            };
        /**
         * Registers a Context Provider to monitor context information.
         * Update set of context parameters when a change occurs.
         * @param contextProviderToken
         */
        /**
         * Registers a Context Provider to monitor context information.
         * Update set of context parameters when a change occurs.
         * @param {?} contextProviderToken
         * @return {?}
         */
        AdaptationController.prototype.registerContextProvider = /**
         * Registers a Context Provider to monitor context information.
         * Update set of context parameters when a change occurs.
         * @param {?} contextProviderToken
         * @return {?}
         */
            function (contextProviderToken) {
                var _this = this;
                // Inject and initialize Context Provider component as a Angular Service
                /** @type {?} */
                var contextProvider = this.injector.get(contextProviderToken);
                // add new component to list of Context Providers
                this.contextProviders.push(contextProvider);
                // get array of observables from all Context Providers
                /** @type {?} */
                var contextProviders = this.contextProviders.map(( /**
                 * @param {?} cp
                 * @return {?}
                 */function (cp) { return cp.getObservable(); }));
                // merge context parameters from all Context Providers
                /** @type {?} */
                var mergedContext$ = rxjs.combineLatest.apply(void 0, __spread(contextProviders)).pipe(operators.map(( /**
                 * @param {?} nestedParams
                 * @return {?}
                 */function (nestedParams) { return [].concat.apply([], __spread(nestedParams)); })));
                // observe changes to merged context and trigger evaluation process
                this.cpSubscription.unsubscribe();
                this.cpSubscription = mergedContext$.subscribe(( /**
                 * @param {?} params
                 * @return {?}
                 */function (params) { return _this.updateContext(params); }));
            };
        /**
         * Register a Rule Provider component
         * Update set of rules when a change occurs
         * @param ruleProviderToken
         */
        /**
         * Register a Rule Provider component
         * Update set of rules when a change occurs
         * @param {?} ruleProviderToken
         * @return {?}
         */
        AdaptationController.prototype.registerRuleProvider = /**
         * Register a Rule Provider component
         * Update set of rules when a change occurs
         * @param {?} ruleProviderToken
         * @return {?}
         */
            function (ruleProviderToken) {
                var _this = this;
                // Inject and initialize Rule Provider component as a Angular Service
                /** @type {?} */
                var ruleProvider = this.injector.get(ruleProviderToken);
                // add new component to list of Rule Providers
                this.ruleProviders.push(ruleProvider);
                // get array of observables from all Rule Providers
                /** @type {?} */
                var ruleProviders = this.ruleProviders.map(( /**
                 * @param {?} rp
                 * @return {?}
                 */function (rp) { return rp.getObservable(); }));
                // merge context parameters from all Rule Providers
                /** @type {?} */
                var mergedRules$ = rxjs.combineLatest.apply(void 0, __spread(ruleProviders)).pipe(operators.map(( /**
                 * @param {?} nestedParams
                 * @return {?}
                 */function (nestedParams) { return [].concat.apply([], __spread(nestedParams)); })));
                // observe changes to merged rules and trigger evaluation process
                this.rpSubscription.unsubscribe();
                this.rpSubscription = mergedRules$.subscribe(( /**
                 * @param {?} rules
                 * @return {?}
                 */function (rules) { return _this.updateRules(rules); }));
            };
        /**
         * Update context parameters and trigger evaluation process
         * @param contextParams
         */
        /**
         * Update context parameters and trigger evaluation process
         * @private
         * @param {?} contextParams
         * @return {?}
         */
        AdaptationController.prototype.updateContext = /**
         * Update context parameters and trigger evaluation process
         * @private
         * @param {?} contextParams
         * @return {?}
         */
            function (contextParams) {
                console.debug("AC: Context updated", contextParams);
                this.context$.next(contextParams);
                this.evaluateAdaptationRules();
            };
        /**
         * Update adaptation rules and trigger evaluation process
         * @param rules
         */
        /**
         * Update adaptation rules and trigger evaluation process
         * @private
         * @param {?} rules
         * @return {?}
         */
        AdaptationController.prototype.updateRules = /**
         * Update adaptation rules and trigger evaluation process
         * @private
         * @param {?} rules
         * @return {?}
         */
            function (rules) {
                console.debug("AC: Rules updated", rules);
                this.rules$.next(rules);
                this.evaluateAdaptationRules();
            };
        /**
         * Start evaluation process, if a Rule Evaluator is specified
         */
        /**
         * Start evaluation process, if a Rule Evaluator is specified
         * @private
         * @return {?}
         */
        AdaptationController.prototype.evaluateAdaptationRules = /**
         * Start evaluation process, if a Rule Evaluator is specified
         * @private
         * @return {?}
         */
            function () {
                var _this = this;
                if (this.ruleEvaluator)
                    this.ruleEvaluator.evaluate(this.getRules(), this.getContext(), ( /**
                     * @param {?} action
                     * @return {?}
                     */function (action) { return _this.fireAdaptation(action); }));
                else
                    console.log("AUI: Evaluation could not be started. No RuleEvaluator set.");
            };
        /**
         * Fire an Adaptation Action
         * Notify observers (Adaptive Widgets) about an Adaptation Action
         * @param action
         */
        /**
         * Fire an Adaptation Action
         * Notify observers (Adaptive Widgets) about an Adaptation Action
         * @private
         * @param {?} action
         * @return {?}
         */
        AdaptationController.prototype.fireAdaptation = /**
         * Fire an Adaptation Action
         * Notify observers (Adaptive Widgets) about an Adaptation Action
         * @private
         * @param {?} action
         * @return {?}
         */
            function (action) {
                this.adaptationActions$.next(action);
                console.debug("Adaptation Controller: AdaptationAction triggered", action);
            };
        Object.defineProperty(AdaptationController.prototype, "AdaptationActions", {
            /** An observable stream of Adaptation Actions */
            get: /**
             * An observable stream of Adaptation Actions
             * @return {?}
             */ function () {
                return this.adaptationActions$.asObservable();
            },
            enumerable: true,
            configurable: true
        });
        /** Returns current context parameters */
        /**
         * Returns current context parameters
         * @return {?}
         */
        AdaptationController.prototype.getContext = /**
         * Returns current context parameters
         * @return {?}
         */
            function () {
                return this.context$.getValue();
            };
        Object.defineProperty(AdaptationController.prototype, "Context", {
            /** An observable stream of context paramteres */
            get: /**
             * An observable stream of context paramteres
             * @return {?}
             */ function () {
                return this.context$.asObservable();
            },
            enumerable: true,
            configurable: true
        });
        /** Returns current set adaptation rules */
        /**
         * Returns current set adaptation rules
         * @return {?}
         */
        AdaptationController.prototype.getRules = /**
         * Returns current set adaptation rules
         * @return {?}
         */
            function () {
                return this.rules$.getValue();
            };
        AdaptationController.decorators = [
            { type: core.Injectable }
        ];
        /** @nocollapse */
        AdaptationController.ctorParameters = function () {
            return [
                { type: core.Injector }
            ];
        };
        return AdaptationController;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var CoBAUIModule = /** @class */ (function () {
        function CoBAUIModule() {
        }
        CoBAUIModule.decorators = [
            { type: core.NgModule, args: [{
                        declarations: [],
                        imports: [],
                        exports: [],
                        providers: [AdaptationController]
                    },] }
        ];
        return CoBAUIModule;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * This component type is used to evaluate Adaptation Rules based on Context Parameters.
     * It triggers Adaptation Actions (defined in rules), if a rules condition is satisfied.
     * @abstract
     */
    var /**
     * This component type is used to evaluate Adaptation Rules based on Context Parameters.
     * It triggers Adaptation Actions (defined in rules), if a rules condition is satisfied.
     * @abstract
     */ RuleEvaluator = /** @class */ (function () {
        function RuleEvaluator() {
        }
        return RuleEvaluator;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * This component type is used to obtain, manage and provide Adaptation Rules.
     * @abstract
     */
    var /**
     * This component type is used to obtain, manage and provide Adaptation Rules.
     * @abstract
     */ RuleProvider = /** @class */ (function () {
        /**
         * Constructor: Initialize empty set of rules
         */
        function RuleProvider() {
            this.rules = [];
            this.rules$ = new rxjs.BehaviorSubject([]);
        }
        /**
         * Publish rules to observers
         */
        /**
         * Publish rules to observers
         * @protected
         * @return {?}
         */
        RuleProvider.prototype.updateRules = /**
         * Publish rules to observers
         * @protected
         * @return {?}
         */
            function () {
                this.rules$.next(this.rules);
            };
        /**
         * Add a rule to the set of rules and notify observers
         */
        /**
         * Add a rule to the set of rules and notify observers
         * @param {?} rule
         * @return {?}
         */
        RuleProvider.prototype.addRule = /**
         * Add a rule to the set of rules and notify observers
         * @param {?} rule
         * @return {?}
         */
            function (rule) {
                this.rules.push(rule);
                this.rules$.next(this.rules);
            };
        /**
         * Remove a rule with given name and notify observers
         */
        /**
         * Remove a rule with given name and notify observers
         * @param {?} name
         * @return {?}
         */
        RuleProvider.prototype.removeRule = /**
         * Remove a rule with given name and notify observers
         * @param {?} name
         * @return {?}
         */
            function (name) {
                this.rules = this.rules.filter(( /**
                 * @param {?} rule
                 * @return {?}
                 */function (rule) { return rule.name !== name; }));
                this.rules$.next(this.rules);
            };
        /**
         * Return observable stream of rules that can be subscribed to by observers
         */
        /**
         * Return observable stream of rules that can be subscribed to by observers
         * @return {?}
         */
        RuleProvider.prototype.getObservable = /**
         * Return observable stream of rules that can be subscribed to by observers
         * @return {?}
         */
            function () {
                return this.rules$.asObservable();
            };
        return RuleProvider;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @enum {string} */
    var UpdateMethod = {
        EVENT_BASED: 'event-based',
        INTERVAL: 'interval',
    };
    /**
     * Base Class for "Context Provider" components.
     * Context Providers obtain information about the context of use (user, platform, environment) and report changes.
     * Provides base structure and functionality
     * @abstract
     */
    var /**
     * Base Class for "Context Provider" components.
     * Context Providers obtain information about the context of use (user, platform, environment) and report changes.
     * Provides base structure and functionality
     * @abstract
     */ ContextProvider = /** @class */ (function () {
        /**
         * Constructor injects an Adaptation Controller component and receives configuration options
         * If updateMethod is chosen to be interval based, setup update interval
         * @param method
         * @param time
         */
        function ContextProvider(method, time) {
            if (time === void 0) {
                time = 10000;
            }
            var _this = this;
            /**
             * Holds array of context parameters
             */
            this.contextParams = [];
            /**
             * An observable stream, that can be subscribed to
             */
            this.contextParams$ = new rxjs.BehaviorSubject([]);
            // setup update interval
            if (method == UpdateMethod.INTERVAL) {
                /** @type {?} */
                var updateInterval = ( /**
                 * @return {?}
                 */function () {
                    _this.updateContext();
                });
                setInterval(updateInterval, time);
            }
        }
        /**
         * Register a new context parameter to this Context Provider
         * @param newParam
         */
        /**
         * Register a new context parameter to this Context Provider
         * @protected
         * @param {?} newParam
         * @return {?}
         */
        ContextProvider.prototype.addContextParam = /**
         * Register a new context parameter to this Context Provider
         * @protected
         * @param {?} newParam
         * @return {?}
         */
            function (newParam) {
                if (this.contextParams.filter(( /**
                 * @param {?} param
                 * @return {?}
                 */function (param) { return param.key === newParam.key; }))) {
                    this.contextParams.push(newParam);
                }
            };
        /**
         * Remove context parameter with given key from this Context Provider
         * @param key
         */
        /**
         * Remove context parameter with given key from this Context Provider
         * @protected
         * @param {?} key
         * @return {?}
         */
        ContextProvider.prototype.removeContextParam = /**
         * Remove context parameter with given key from this Context Provider
         * @protected
         * @param {?} key
         * @return {?}
         */
            function (key) {
                this.contextParams = this.contextParams.filter(( /**
                 * @param {?} param
                 * @return {?}
                 */function (param) { return param.key !== key; }));
            };
        /**
         * Returns a context parameter with given key
         * @param key
         */
        /**
         * Returns a context parameter with given key
         * @protected
         * @param {?} key
         * @return {?}
         */
        ContextProvider.prototype.getContextParam = /**
         * Returns a context parameter with given key
         * @protected
         * @param {?} key
         * @return {?}
         */
            function (key) {
                return this.contextParams.find(( /**
                 * @param {?} param
                 * @return {?}
                 */function (param) { return param.key == key; }));
            };
        /**
         * Modifies the value of a context parameter with given key
         * Notice: Does not implicitly report context update to observers.
         * @param key
         * @param newValue
         */
        /**
         * Modifies the value of a context parameter with given key
         * Notice: Does not implicitly report context update to observers.
         * @protected
         * @param {?} key
         * @param {?} newValue
         * @return {?}
         */
        ContextProvider.prototype.modifyContextParam = /**
         * Modifies the value of a context parameter with given key
         * Notice: Does not implicitly report context update to observers.
         * @protected
         * @param {?} key
         * @param {?} newValue
         * @return {?}
         */
            function (key, newValue) {
                /** @type {?} */
                var oldParam = this.getContextParam(key);
                if (oldParam) {
                    this.removeContextParam(key);
                    this.addContextParam({ contextOfUse: oldParam.contextOfUse, type: oldParam.type, key: oldParam.key, value: newValue });
                }
            };
        /** Return observable stream of context information */
        /**
         * Return observable stream of context information
         * @return {?}
         */
        ContextProvider.prototype.getObservable = /**
         * Return observable stream of context information
         * @return {?}
         */
            function () {
                return this.contextParams$.asObservable();
            };
        /** Update and emit context information to observers */
        /**
         * Update and emit context information to observers
         * @protected
         * @return {?}
         */
        ContextProvider.prototype.updateContext = /**
         * Update and emit context information to observers
         * @protected
         * @return {?}
         */
            function () {
                this.contextParams$.next(this.contextParams);
            };
        return ContextProvider;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * An AdaptiveWidget is a UI Component extended by adaptive functionality.
     * It defines and implements adaptations for itself, encapsulating the adaptation logic to the outside.
     * Adaptations can take various effects on the Widget, like changing its layout, modality or behaviour.
     * @abstract
     */
    var AdaptiveWidget = /** @class */ (function () {
        /**
         * The Adaptation Controller is injected to enable and control dynamic context adaptation at runtime.
         * @param adaptationController
         * @param name
         */
        function AdaptiveWidget(adaptationController, name) {
            this.adaptationController = adaptationController;
            this.name = name;
            /**
             * The namespace specifies a scope in which the widget adaptations are triggered.
             * This allows for more control over applying rules to certain instances or groups of this widget.
             */
            this.namespace = '';
            this.setupAdaptivity();
        }
        /**
         * @return {?}
         */
        AdaptiveWidget.prototype.ngOnInit = /**
         * @return {?}
         */
            function () {
            };
        /**
         * @return {?}
         */
        AdaptiveWidget.prototype.ngOnDestroy = /**
         * @return {?}
         */
            function () {
            };
        /**
         * Register to the Adaptation Controller and listen for Adaptation Actions.
         * Check if Adaptation Action is addressed to this widget and perform given adaptation.
         */
        /**
         * Register to the Adaptation Controller and listen for Adaptation Actions.
         * Check if Adaptation Action is addressed to this widget and perform given adaptation.
         * @private
         * @return {?}
         */
        AdaptiveWidget.prototype.setupAdaptivity = /**
         * Register to the Adaptation Controller and listen for Adaptation Actions.
         * Check if Adaptation Action is addressed to this widget and perform given adaptation.
         * @private
         * @return {?}
         */
            function () {
                var _this = this;
                this.adaptationController.AdaptationActions
                    .pipe(operators.filter(( /**
             * @param {?} action
             * @return {?}
             */function (action) { return action.target === _this.name && (action.scope.length === 0 || action.scope.includes(_this.namespace)); })))
                    .subscribe(( /**
             * @param {?} action
             * @return {?}
             */function (action) { return _this.adapt(action); }));
            };
        /**
         * Here the adaptation capabilities of the widget are implemented.
         * This function is meant to be overridden by inheriting widgets.
         * @param action
         */
        /**
         * Here the adaptation capabilities of the widget are implemented.
         * This function is meant to be overridden by inheriting widgets.
         * @param {?} action
         * @return {?}
         */
        AdaptiveWidget.prototype.adapt = /**
         * Here the adaptation capabilities of the widget are implemented.
         * This function is meant to be overridden by inheriting widgets.
         * @param {?} action
         * @return {?}
         */
            function (action) {
                console.debug("Widget (" + this.name + "/" + this.namespace + "): Action '" + action.name + "' performed", action);
            };
        AdaptiveWidget.propDecorators = {
            namespace: [{ type: core.Input, args: ['aui-namespace',] }]
        };
        return AdaptiveWidget;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    exports.CoBAUIModule = CoBAUIModule;
    exports.AdaptationController = AdaptationController;
    exports.RuleEvaluator = RuleEvaluator;
    exports.RuleProvider = RuleProvider;
    exports.UpdateMethod = UpdateMethod;
    exports.ContextProvider = ContextProvider;
    exports.AdaptiveWidget = AdaptiveWidget;

    Object.defineProperty(exports, '__esModule', { value: true });

})));

//# sourceMappingURL=cobaui.umd.js.map