import { ContextParam } from './context-param.interface';
export declare enum UpdateMethod {
    EVENT_BASED = "event-based",
    INTERVAL = "interval"
}
/**
 * Base Class for "Context Provider" components.
 * Context Providers obtain information about the context of use (user, platform, environment) and report changes.
 * Provides base structure and functionality
 */
export declare abstract class ContextProvider {
    /** Holds array of context parameters */
    private contextParams;
    /** An observable stream, that can be subscribed to */
    private contextParams$;
    /**
     * Constructor injects an Adaptation Controller component and receives configuration options
     * If updateMethod is chosen to be interval based, setup update interval
     * @param method
     * @param time
     */
    protected constructor(method: UpdateMethod, time?: number);
    /**
     * Register a new context parameter to this Context Provider
     * @param newParam
     */
    protected addContextParam(newParam: ContextParam): void;
    /**
     * Remove context parameter with given key from this Context Provider
     * @param key
     */
    protected removeContextParam(key: string): void;
    /**
     * Returns a context parameter with given key
     * @param key
     */
    protected getContextParam(key: string): ContextParam;
    /**
     * Modifies the value of a context parameter with given key
     * Notice: Does not implicitly report context update to observers.
     * @param key
     * @param newValue
     */
    protected modifyContextParam(key: string, newValue: any): void;
    /** Return observable stream of context information */
    getObservable(): import("rxjs").Observable<ContextParam[]>;
    /** Update and emit context information to observers */
    protected updateContext(): void;
}
