import { AdaptationRule } from './adaptation-rule.interface';
/**
 * This component type is used to obtain, manage and provide Adaptation Rules.
 */
export declare abstract class RuleProvider {
    /** Set of adaptation rules */
    protected rules: AdaptationRule[];
    /** Observable set of adaptation rules */
    private rules$;
    /**
     * Constructor: Initialize empty set of rules
     */
    protected constructor();
    /**
     * Publish rules to observers
     */
    protected updateRules(): void;
    /**
     * Add a rule to the set of rules and notify observers
     */
    addRule(rule: AdaptationRule): void;
    /**
     * Remove a rule with given name and notify observers
     */
    removeRule(name: string): void;
    /**
     * Return observable stream of rules that can be subscribed to by observers
     */
    getObservable(): import("rxjs").Observable<AdaptationRule[]>;
}
