import { OnDestroy, OnInit } from '@angular/core';
import { AdaptationController } from '../core/adaptation-controller';
import { AdaptationAction } from './adaptation-action.interface';
/**
 * An AdaptiveWidget is a UI Component extended by adaptive functionality.
 * It defines and implements adaptations for itself, encapsulating the adaptation logic to the outside.
 * Adaptations can take various effects on the Widget, like changing its layout, modality or behaviour.
 */
export declare abstract class AdaptiveWidget implements OnInit, OnDestroy {
    private adaptationController;
    private name;
    /**
     * The namespace specifies a scope in which the widget adaptations are triggered.
     * This allows for more control over applying rules to certain instances or groups of this widget.
     */
    namespace: string;
    /**
     * The Adaptation Controller is injected to enable and control dynamic context adaptation at runtime.
     * @param adaptationController
     * @param name
     */
    protected constructor(adaptationController: AdaptationController, name: string);
    ngOnInit(): void;
    ngOnDestroy(): void;
    /**
     * Register to the Adaptation Controller and listen for Adaptation Actions.
     * Check if Adaptation Action is addressed to this widget and perform given adaptation.
     */
    private setupAdaptivity;
    /**
     * Here the adaptation capabilities of the widget are implemented.
     * This function is meant to be overridden by inheriting widgets.
     * @param action
     */
    adapt(action: AdaptationAction): void;
}
